<?php

$plugin = array(
  'title' => t('REST'),
  'description' => t('Makes GET requests to receive JSON data at a configurable URL in order to retrieve the contents of the repository.'),
  'handler' => array(
    'class' => 'RepoviewREST',
  ),
  'backends' => array('all'),
);

class RepoviewREST implements RepoviewRepoInterface {

  public $server = '';
  
  public function __construct($settings) {
    if (isset($settings['server'])) {
      $this->server = $settings['server'];
    }
  }

  /**
   * Render the settings form for this item.
   */
  public function form() {
    $form = array();
    $form['server'] = array(
      '#title' => t('Server URL'),
      '#description' => t('The URL of the server from which to retrieve the repository contents.'),
      '#type' => 'textfield',
      '#default_value' => $this->server,
    );
    return $form;
  }

  /** 
   * Get a representation of an Object within the Git Repository.
   *
   * @param string $repo_location
   *   The path to the actual repository on disk.
   * @param string $path
   *   The path withihin the repository for which we are trying to retrieve an object.
   * @param string $branch
   *   The branch on which we are trying to view this object.
   *   TODO: This is here out of a vague suspicion that it'll be critical for svn
   *   (if this is abstracted that far).  Should it be?
   * @param string $commit
   *   The revision at which we are trying to view this object.
   *
   * @return mixed
   *   On failure, FALSE.
   *   On success, a stdClass object whose attributes are as follows:
   *     @code
   *        // The name of this element.
   *        $data->name '';
   *        // The type of this element, either file or directory.
   *        $data->type '';
   *        // The contents of this element, for files the content of the file for
   *        // directories a multidimmenstional array.
   *        $data->contents '';
   *     @endcode
   */
  public function getObject($repo_location, $path, $branch, $commit) {
    $params = array();
    $params['objectPath'] = ($path == '' ? '/' : $path);
    $params['path'] = $repo_location;
    $data = $this->sendRequest('getObject', $params);
    if ($data->type == 'file') {
      $data->contents = base64_decode($data->contents);
    }
    else if ($data->type == 'directory') {
      $data->contents = (array) $data->contents;
      foreach ($data->contents as &$item) {
        $item = (array) $item;
      }
      sort($data->contents);
    }
    return $data;
  }


  /** 
   * Get a representation of an Object within the Git Repository.
   *
   * @param string $repo_location
   *   The path to the actual repository on disk.
   * @param string $branch
   *   The branch on which we are trying to view this object.
   *   TODO: This is here out of a vague suspicion that it'll be critical for svn
   *   (if this is abstracted that far).  Should it be?
   * @param string $commit
   *   TODO: retitle this revision?
   *   The revision at which we are trying to view this object.
   *
   * @return string
   *   The contents of the commit.
   */
  public function getCommit($repo_location, $branch, $commit) {
    $data['path'] = $repo_location;
    $data['branch'] = $branch;
    $data['commit'] = $commit;
    $data = $this->sendRequest('getCommit', $data);
    return $data;
  }

  /**
   * Send a request to a remote, a convenience wrapper around drupal_http_request().
   */
  private function sendRequest($resource, $params = array()) {
    $query = '';
    $query_params = array();
    if (count($params)) {
      foreach ($params as $name => $value) {
        $query_params[] = $name . '=' . $value;
      }
    }
    $query = implode('&', $query_params);
    $url = $this->server . '/' . $resource . '?' . $query;
    $header = array('Content-Type' => 'text/json');
    $response_data = drupal_http_request($url, $headers, 'GET');
    if (!$data = json_decode($response_data->data)) {
      drupal_set_message(t('The remote server used to load the repository contents did not return valid data.  Please contact an administrator if this problem persists.'), 'error');
      watchdog('repoview', 'The URL @uri did not return valid JSON.', array('@path' => $url), WATCHDOG_ERROR); 
    }
    return $data;
  }
}
